import pandas as pd
from janome.tokenizer import Tokenizer

from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.svm import SVC
from sklearn.externals import joblib

from sklearn.model_selection import train_test_split


def pkl_save(pkl, pkl_path):
  joblib.dump(pkl,  pkl_path)

def pkl_load(pkl_path):
  return joblib.load(pkl_path)

def parse_text(text, tokenizer):
  tokens=tokenizer.tokenize(text)
  res=[]

  for token in tokens:
    pos = token.part_of_speech.split(",")[0]
    #if not ps in ['名詞', '動詞', '形容詞', '記号']: 
    if not pos in [b'\xe5\x90\x8d\xe8\xa9\x9e'.decode("utf-8"), b'\xe5\x8b\x95\xe8\xa9\x9e'.decode("utf-8"), b'\xe5\xbd\xa2\xe5\xae\xb9\xe8\xa9\x9e'.decode("utf-8"), b'\xe8\xa8\x98\xe5\x8f\xb7'.decode("utf-8")]: 
       continue # 他の品詞は無視

    pos = token.part_of_speech.split(",")[1]
    if pos in [b'\xe6\x95\xb0'.decode("utf-8")]: 
      continue # 数以外は無視
    
    w=token.base_form
    # 一部の記号は無視
    if w=="!" or w=="!?" or w=="#" or w=="\n": continue
    
    res.append(w)

  if len(res) < 1:
    res.append(text)

  return res


def training(file, vect_path, clf_path):
  #Janomeを使って形態素解析のインスタンスを生成
  ja_tokenizer = Tokenizer()

  # PandasにてCSVファイルの読み取りDataFrameの形式で保存
  df = pd.read_csv(file, encoding="utf-8")

  # DataFrameのtitleのカラムをループし、日本語タイトルのクレンジング
  train_list = []
  for text in df["title"]:
    res = parse_text(text, ja_tokenizer)
    train_list.append(' '.join(res))
  
  # 日本語データのベクトル化
  vect = TfidfVectorizer()
  vect.fit(train_list)
  X = vect.transform(train_list)
  Y = df["category"].values

  # SVMでのフィッティング、モデル作成
  clf = SVC(kernel='linear', probability=True)    
  clf.fit(X, Y)
  
  # モデルの保存
  pkl_save(vect, vect_path)
  pkl_save(clf, clf_path)
  
  # 検証用として、DataFrameへクレンジング後のものを追加
  df["mod title"] = train_list
  return df


def predicting(predict_list, vect_path, clf_path):
  #Janomeを使って形態素解析のインスタンスを生成
  ja_tokenizer = Tokenizer()
  
  # モデルの読み込み
  vect = pkl_load(vect_path)
  clf = pkl_load(clf_path)
  
  # インプットがテキストの場合、リストへ変更
  if isinstance(predict_list, str):
    predict_list = [predict_list]

  # 予測する日本語タイトルのクレンジング
  alt_predict_list=[]
  for text in predict_list:
    res = parse_text(text, ja_tokenizer)
    alt_predict_list.append(' '.join(res))
  
  # 日本語データのベクトル化
  X = vect.transform(alt_predict_list)

  # SVMでの予測
  result_list = clf.predict(X)

  # スコアの取得
  predict_proba_test_scores = clf.predict_proba(X) 
  score_list = list(map(lambda x: max(x), predict_proba_test_scores))
  
  # 検証用にDataFrameの作成
  df = pd.DataFrame(predict_list, columns=['title'])
  df['mod title'] = alt_predict_list
  df['result'] = result_list
  df['score'] = score_list

  return df


def predicting_for_uipath(predict_text, vect_path, clf_path):
  df = predicting(predict_text, vect_path, clf_path)
  return [df['result'].iloc[0], str(df['score'].iloc[0])]


if __name__ == '__main__':
  file = "./thinkit_category.csv"
  vect_path = "classifier_vect.pkl"
  clf_path = "classifier_clf.pkl"

  df = training(file, vect_path, clf_path)
  print(df)

  #predict_list = [
  #  "デバイス不要の完全没入型VRシステムが登場", # 業務アプリ
  #  "Cloud Foundry Summitはエコシステムの拡がりを感じるカンファレンス", # ITインフラ
  #  "ユーザビリティの判断基準って何？", # システム開発
  #  "選択肢が広がるBI製品", # 業務アプリ
  #  "3種類のノードをセットアップ！" # 運用管理
  #]
  
  #df_res = predicting(predict_list, vect_path, clf_path)
  #print(df_res)

  #result = predicting_for_uipath("デバイス不要の完全没入型VRシステムが登場", vect_path, clf_path)
  #print(result)

